use blake2::Digest;
use std::io::{BufReader, Read};

use blake2::Blake2s256;
use borsh::{to_vec, BorshDeserialize, BorshSerialize};
use xz2::bufread::XzEncoder;

#[derive(BorshSerialize, BorshDeserialize, PartialEq, Debug)]
enum Test {
    A(String),
}

#[derive(BorshSerialize, BorshDeserialize, PartialEq, Debug)]
struct A {
    x: u64,
    y: String,
    z: Test,
}

#[test]
fn test_simple_struct() {
    let a = A {
        x: 3301,
        y: "liber primus".to_string(),
        z: Test::A("test".to_string()),
    };
    let encoded_a = to_vec(&a).unwrap();
    let decoded_a = A::try_from_slice(&encoded_a).unwrap();
    assert_eq!(a, decoded_a);
}

fn main() {
    let a = A {
        x: 3301,
        y: "Justo nec ultrices dui sapien eget mi. Vestibulum sed arcu non odio euismod lacinia at. Scelerisque eleifend donec pretium vulputate sapien. Faucibus a pellentesque sit amet. Enim eu turpis egestas pretium aenean pharetra magna. Sollicitudin ac orci phasellus egestas tellus rutrum tellus. Faucibus in ornare quam viverra orci sagittis eu. Suspendisse sed nisi lacus sed viverra tellus in hac habitasse. Mauris augue neque gravida in fermentum. Quam adipiscing vitae proin sagittis nisl rhoncus mattis. Non arcu risus quis varius quam quisque. Id venenatis a condimentum vitae sapien pellentesque habitant morbi. Sapien eget mi proin sed libero. Varius vel pharetra vel turpis nunc. Duis tristique sollicitudin nibh sit amet. Elementum tempus egestas sed sed risus pretium quam.".to_string(),
        z: Test::A("Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.".to_string()),
    };
    let encoded_a = to_vec(&a).unwrap();
    let decoded_a = A::try_from_slice(&encoded_a).unwrap();
    assert!(a == decoded_a); // Double check

    let mut bufread = BufReader::new(&encoded_a[..]);
    let mut compressed_lzma_rs = vec![];
    lzma_rs::lzma_compress(&mut bufread, &mut compressed_lzma_rs).unwrap();

    let bufread = BufReader::new(&encoded_a[..]);
    let mut compressed_xz = vec![];
    XzEncoder::new(bufread, 9)
        .read_to_end(&mut compressed_xz)
        .unwrap();

    let bufread = BufReader::new(&encoded_a[..]);
    let compressed_zstd = zstd::encode_all(bufread, 22).unwrap();

    println!("Size of encoded_a: {}", encoded_a.len());
    println!("Size of compressed_lzma_rs: {}", compressed_lzma_rs.len());
    println!("Size of compressed_zstd: {}", compressed_zstd.len());
    println!("Size of compressed_xz: {}", compressed_xz.len());

    let mut hasher = Blake2s256::new();
    hasher.update(&compressed_zstd); // dd7e168f2993e734772acab1fb8bdcf65b0302094350719885b6be6779e3768f
    let hash = hasher.finalize();
    println!("Hash of encoded_a: {:x}", hash);
    assert!(
        format!("{:x}", hash) == "dd7e168f2993e734772acab1fb8bdcf65b0302094350719885b6be6779e3768f"
    );
}
